@@ -1,12 +1,13 @@
 #![feature(test)]
 
-extern crate gimli;
 extern crate test;
 
-use gimli::{AttributeValue, DebugAbbrev, DebugAranges, DebugInfo, DebugLine, DebugLineOffset,
-            DebugLoc, DebugLocLists, DebugPubNames, DebugPubTypes, DebugRanges, DebugRngLists,
-            EntriesTreeNode, Expression, Format, LittleEndian, LocationLists, Operation,
-            RangeLists, Reader};
+use gimli::{
+    AttributeValue, DebugAbbrev, DebugAddr, DebugAddrBase, DebugAranges, DebugInfo, DebugLine,
+    DebugLineOffset, DebugLoc, DebugLocLists, DebugPubNames, DebugPubTypes, DebugRanges,
+    DebugRngLists, Encoding, EndianSlice, EntriesTreeNode, Expression, LittleEndian, LocationLists,
+    Operation, RangeLists, Reader, ReaderOffset,
+};
 use std::env;
 use std::fs::File;
 use std::io::Read;
@@ -48,10 +49,14 @@ fn bench_parsing_debug_abbrev(b: &mut test::Bencher) {
 }
 
 #[inline]
-fn impl_bench_parsing_debug_info<R: Reader>(debug_info: DebugInfo<R>, debug_abbrev: DebugAbbrev<R>) {
+fn impl_bench_parsing_debug_info<R: Reader>(
+    debug_info: DebugInfo<R>,
+    debug_abbrev: DebugAbbrev<R>,
+) {
     let mut iter = debug_info.units();
     while let Some(unit) = iter.next().expect("Should parse compilation unit") {
-        let abbrevs = unit.abbreviations(&debug_abbrev)
+        let abbrevs = unit
+            .abbreviations(&debug_abbrev)
             .expect("Should parse abbreviations");
 
         let mut cursor = unit.entries(&abbrevs);
@@ -102,10 +107,12 @@ fn bench_parsing_debug_info_tree(b: &mut test::Bencher) {
 
         let mut iter = debug_info.units();
         while let Some(unit) = iter.next().expect("Should parse compilation unit") {
-            let abbrevs = unit.abbreviations(&debug_abbrev)
+            let abbrevs = unit
+                .abbreviations(&debug_abbrev)
                 .expect("Should parse abbreviations");
 
-            let mut tree = unit.entries_tree(&abbrevs, None)
+            let mut tree = unit
+                .entries_tree(&abbrevs, None)
                 .expect("Should have entries tree");
             let root = tree.root().expect("Should parse root entry");
             parse_debug_info_tree(root);
@@ -182,9 +189,12 @@ fn bench_parsing_line_number_program_opcodes(b: &mut test::Bencher) {
             .expect("Should parse line number program header");
         let header = program.header();
 
-        let mut opcodes = header.opcodes();
-        while let Some(opcode) = opcodes.next_opcode(header).expect("Should parse opcode") {
-            test::black_box(opcode);
+        let mut instructions = header.instructions();
+        while let Some(instruction) = instructions
+            .next_instruction(header)
+            .expect("Should parse instruction")
+        {
+            test::black_box(instruction);
         }
     });
 }
@@ -200,7 +210,8 @@ fn bench_executing_line_number_programs(b: &mut test::Bencher) {
             .expect("Should parse line number program header");
 
         let mut rows = program.rows();
-        while let Some(row) = rows.next_row()
+        while let Some(row) = rows
+            .next_row()
             .expect("Should parse and execute all rows in the line number program")
         {
             test::black_box(row);
@@ -216,16 +227,20 @@ fn bench_parsing_debug_loc(b: &mut test::Bencher) {
     let debug_abbrev = read_section("debug_abbrev");
     let debug_abbrev = DebugAbbrev::new(&debug_abbrev, LittleEndian);
 
+    let debug_addr = DebugAddr::from(EndianSlice::new(&[], LittleEndian));
+    let debug_addr_base = DebugAddrBase(0);
+
     let debug_loc = read_section("debug_loc");
     let debug_loc = DebugLoc::new(&debug_loc, LittleEndian);
     let debug_loclists = DebugLocLists::new(&[], LittleEndian);
-    let loclists = LocationLists::new(debug_loc, debug_loclists).expect("Should parse loclists");
+    let loclists = LocationLists::new(debug_loc, debug_loclists);
 
     let mut offsets = Vec::new();
 
     let mut iter = debug_info.units();
     while let Some(unit) = iter.next().expect("Should parse compilation unit") {
-        let abbrevs = unit.abbreviations(&debug_abbrev)
+        let abbrevs = unit
+            .abbreviations(&debug_abbrev)
             .expect("Should parse abbreviations");
 
         let mut cursor = unit.entries(&abbrevs);
@@ -248,16 +263,16 @@ fn bench_parsing_debug_loc(b: &mut test::Bencher) {
             let mut attrs = entry.attrs();
             while let Some(attr) = attrs.next().expect("Should parse entry's attribute") {
                 if let gimli::AttributeValue::LocationListsRef(offset) = attr.value() {
-                    offsets.push((offset, unit.version(), unit.address_size(), low_pc));
+                    offsets.push((offset, unit.encoding(), low_pc));
                 }
             }
         }
     }
 
     b.iter(|| {
-        for &(offset, version, address_size, base_address) in &*offsets {
+        for &(offset, encoding, base_address) in &*offsets {
             let mut locs = loclists
-                .locations(offset, version, address_size, base_address)
+                .locations(offset, encoding, base_address, &debug_addr, debug_addr_base)
                 .expect("Should parse locations OK");
             while let Some(loc) = locs.next().expect("Should parse next location") {
                 test::black_box(loc);
@@ -274,16 +289,20 @@ fn bench_parsing_debug_ranges(b: &mut test::Bencher) {
     let debug_abbrev = read_section("debug_abbrev");
     let debug_abbrev = DebugAbbrev::new(&debug_abbrev, LittleEndian);
 
+    let debug_addr = DebugAddr::from(EndianSlice::new(&[], LittleEndian));
+    let debug_addr_base = DebugAddrBase(0);
+
     let debug_ranges = read_section("debug_ranges");
     let debug_ranges = DebugRanges::new(&debug_ranges, LittleEndian);
     let debug_rnglists = DebugRngLists::new(&[], LittleEndian);
-    let rnglists = RangeLists::new(debug_ranges, debug_rnglists).expect("Should parse rnglists");
+    let rnglists = RangeLists::new(debug_ranges, debug_rnglists);
 
     let mut offsets = Vec::new();
 
     let mut iter = debug_info.units();
     while let Some(unit) = iter.next().expect("Should parse compilation unit") {
-        let abbrevs = unit.abbreviations(&debug_abbrev)
+        let abbrevs = unit
+            .abbreviations(&debug_abbrev)
             .expect("Should parse abbreviations");
 
         let mut cursor = unit.entries(&abbrevs);
@@ -306,16 +325,16 @@ fn bench_parsing_debug_ranges(b: &mut test::Bencher) {
             let mut attrs = entry.attrs();
             while let Some(attr) = attrs.next().expect("Should parse entry's attribute") {
                 if let gimli::AttributeValue::RangeListsRef(offset) = attr.value() {
-                    offsets.push((offset, unit.version(), unit.address_size(), low_pc));
+                    offsets.push((offset, unit.encoding(), low_pc));
                 }
             }
         }
     }
 
     b.iter(|| {
-        for &(offset, version, address_size, base_address) in &*offsets {
+        for &(offset, encoding, base_address) in &*offsets {
             let mut ranges = rnglists
-                .ranges(offset, version, address_size, base_address)
+                .ranges(offset, encoding, base_address, &debug_addr, debug_addr_base)
                 .expect("Should parse ranges OK");
             while let Some(range) = ranges.next().expect("Should parse next range") {
                 test::black_box(range);
@@ -327,12 +346,13 @@ fn bench_parsing_debug_ranges(b: &mut test::Bencher) {
 fn debug_info_expressions<R: Reader>(
     debug_info: &DebugInfo<R>,
     debug_abbrev: &DebugAbbrev<R>,
-) -> Vec<(Expression<R>, u8, Format)> {
+) -> Vec<(Expression<R>, Encoding)> {
     let mut expressions = Vec::new();
 
     let mut iter = debug_info.units();
     while let Some(unit) = iter.next().expect("Should parse compilation unit") {
-        let abbrevs = unit.abbreviations(debug_abbrev)
+        let abbrevs = unit
+            .abbreviations(debug_abbrev)
             .expect("Should parse abbreviations");
 
         let mut cursor = unit.entries(&abbrevs);
@@ -340,7 +360,7 @@ fn debug_info_expressions<R: Reader>(
             let mut attrs = entry.attrs();
             while let Some(attr) = attrs.next().expect("Should parse entry's attribute") {
                 if let AttributeValue::Exprloc(expression) = attr.value() {
-                    expressions.push((expression, unit.address_size(), unit.format()));
+                    expressions.push((expression, unit.encoding()));
                 }
             }
         }
@@ -360,11 +380,10 @@ fn bench_parsing_debug_info_expressions(b: &mut test::Bencher) {
     let expressions = debug_info_expressions(&debug_info, &debug_abbrev);
 
     b.iter(|| {
-        for &(expression, address_size, format) in &*expressions {
+        for &(expression, encoding) in &*expressions {
             let mut pc = expression.0;
             while !pc.is_empty() {
-                Operation::parse(&mut pc, &expression.0, address_size, format)
-                    .expect("Should parse operation");
+                Operation::parse(&mut pc, &expression.0, encoding).expect("Should parse operation");
             }
         }
     });
@@ -381,8 +400,8 @@ fn bench_evaluating_debug_info_expressions(b: &mut test::Bencher) {
     let expressions = debug_info_expressions(&debug_info, &debug_abbrev);
 
     b.iter(|| {
-        for &(expression, address_size, format) in &*expressions {
-            let mut eval = expression.evaluation(address_size, format);
+        for &(expression, encoding) in &*expressions {
+            let mut eval = expression.evaluation(encoding);
             eval.set_initial_value(0);
             let result = eval.evaluate().expect("Should evaluate expression");
             test::black_box(result);
@@ -393,13 +412,17 @@ fn bench_evaluating_debug_info_expressions(b: &mut test::Bencher) {
 fn debug_loc_expressions<R: Reader>(
     debug_info: &DebugInfo<R>,
     debug_abbrev: &DebugAbbrev<R>,
+    debug_addr: &DebugAddr<R>,
     loclists: &LocationLists<R>,
-) -> Vec<(Expression<R>, u8, Format)> {
+) -> Vec<(Expression<R>, Encoding)> {
+    let debug_addr_base = DebugAddrBase(R::Offset::from_u8(0));
+
     let mut expressions = Vec::new();
 
     let mut iter = debug_info.units();
     while let Some(unit) = iter.next().expect("Should parse compilation unit") {
-        let abbrevs = unit.abbreviations(debug_abbrev)
+        let abbrevs = unit
+            .abbreviations(debug_abbrev)
             .expect("Should parse abbreviations");
 
         let mut cursor = unit.entries(&abbrevs);
@@ -423,10 +446,10 @@ fn debug_loc_expressions<R: Reader>(
             while let Some(attr) = attrs.next().expect("Should parse entry's attribute") {
                 if let gimli::AttributeValue::LocationListsRef(offset) = attr.value() {
                     let mut locs = loclists
-                        .locations(offset, unit.version(), unit.address_size(), low_pc)
+                        .locations(offset, unit.encoding(), low_pc, debug_addr, debug_addr_base)
                         .expect("Should parse locations OK");
                     while let Some(loc) = locs.next().expect("Should parse next location") {
-                        expressions.push((loc.data, unit.address_size(), unit.format()));
+                        expressions.push((loc.data, unit.encoding()));
                     }
                 }
             }
@@ -444,19 +467,20 @@ fn bench_parsing_debug_loc_expressions(b: &mut test::Bencher) {
     let debug_abbrev = read_section("debug_abbrev");
     let debug_abbrev = DebugAbbrev::new(&debug_abbrev, LittleEndian);
 
+    let debug_addr = DebugAddr::from(EndianSlice::new(&[], LittleEndian));
+
     let debug_loc = read_section("debug_loc");
     let debug_loc = DebugLoc::new(&debug_loc, LittleEndian);
     let debug_loclists = DebugLocLists::new(&[], LittleEndian);
-    let loclists = LocationLists::new(debug_loc, debug_loclists).expect("Should parse loclists");
+    let loclists = LocationLists::new(debug_loc, debug_loclists);
 
-    let expressions = debug_loc_expressions(&debug_info, &debug_abbrev, &loclists);
+    let expressions = debug_loc_expressions(&debug_info, &debug_abbrev, &debug_addr, &loclists);
 
     b.iter(|| {
-        for &(expression, address_size, format) in &*expressions {
+        for &(expression, encoding) in &*expressions {
             let mut pc = expression.0;
             while !pc.is_empty() {
-                Operation::parse(&mut pc, &expression.0, address_size, format)
-                    .expect("Should parse operation");
+                Operation::parse(&mut pc, &expression.0, encoding).expect("Should parse operation");
             }
         }
     });
@@ -470,16 +494,18 @@ fn bench_evaluating_debug_loc_expressions(b: &mut test::Bencher) {
     let debug_abbrev = read_section("debug_abbrev");
     let debug_abbrev = DebugAbbrev::new(&debug_abbrev, LittleEndian);
 
+    let debug_addr = DebugAddr::from(EndianSlice::new(&[], LittleEndian));
+
     let debug_loc = read_section("debug_loc");
     let debug_loc = DebugLoc::new(&debug_loc, LittleEndian);
     let debug_loclists = DebugLocLists::new(&[], LittleEndian);
-    let loclists = LocationLists::new(debug_loc, debug_loclists).expect("Should parse loclists");
+    let loclists = LocationLists::new(debug_loc, debug_loclists);
 
-    let expressions = debug_loc_expressions(&debug_info, &debug_abbrev, &loclists);
+    let expressions = debug_loc_expressions(&debug_info, &debug_abbrev, &debug_addr, &loclists);
 
     b.iter(|| {
-        for &(expression, address_size, format) in &*expressions {
-            let mut eval = expression.evaluation(address_size, format);
+        for &(expression, encoding) in &*expressions {
+            let mut eval = expression.evaluation(encoding);
             eval.set_initial_value(0);
             let result = eval.evaluate().expect("Should evaluate expression");
             test::black_box(result);
@@ -490,22 +516,23 @@ fn bench_evaluating_debug_loc_expressions(b: &mut test::Bencher) {
 // See comment above `test_parse_self_eh_frame`.
 #[cfg(target_pointer_width = "64")]
 mod cfi {
-    extern crate fallible_iterator;
-    extern crate gimli;
-    extern crate test;
-
     use super::*;
-    use self::fallible_iterator::FallibleIterator;
+    use fallible_iterator::FallibleIterator;
 
-    use gimli::{BaseAddresses, CieOrFde, EhFrame, FrameDescriptionEntry, LittleEndian,
-                UninitializedUnwindContext, UnwindSection, UnwindTable};
+    use gimli::{
+        BaseAddresses, CieOrFde, EhFrame, FrameDescriptionEntry, LittleEndian,
+        UninitializedUnwindContext, UnwindSection,
+    };
 
     #[bench]
     fn iterate_entries_and_do_not_parse_any_fde(b: &mut test::Bencher) {
         let eh_frame = read_section("eh_frame");
         let eh_frame = EhFrame::new(&eh_frame, LittleEndian);
 
-        let bases = BaseAddresses::default().set_cfi(0).set_data(0).set_text(0);
+        let bases = BaseAddresses::default()
+            .set_eh_frame(0)
+            .set_got(0)
+            .set_text(0);
 
         b.iter(|| {
             let mut entries = eh_frame.entries(&bases);
@@ -520,7 +547,10 @@ mod cfi {
         let eh_frame = read_section("eh_frame");
         let eh_frame = EhFrame::new(&eh_frame, LittleEndian);
 
-        let bases = BaseAddresses::default().set_cfi(0).set_data(0).set_text(0);
+        let bases = BaseAddresses::default()
+            .set_eh_frame(0)
+            .set_got(0)
+            .set_text(0);
 
         b.iter(|| {
             let mut entries = eh_frame.entries(&bases);
@@ -531,7 +561,7 @@ mod cfi {
                     }
                     CieOrFde::Fde(partial) => {
                         let fde = partial
-                            .parse(|offset| eh_frame.cie_from_offset(&bases, offset))
+                            .parse(EhFrame::cie_from_offset)
                             .expect("Should be able to get CIE for FED");
                         test::black_box(fde);
                     }
@@ -545,14 +575,17 @@ mod cfi {
         let eh_frame = read_section("eh_frame");
         let eh_frame = EhFrame::new(&eh_frame, LittleEndian);
 
-        let bases = BaseAddresses::default().set_cfi(0).set_data(0).set_text(0);
+        let bases = BaseAddresses::default()
+            .set_eh_frame(0)
+            .set_got(0)
+            .set_text(0);
 
         b.iter(|| {
             let mut entries = eh_frame.entries(&bases);
             while let Some(entry) = entries.next().expect("Should parse CFI entry OK") {
                 match entry {
                     CieOrFde::Cie(cie) => {
-                        let mut instrs = cie.instructions();
+                        let mut instrs = cie.instructions(&eh_frame, &bases);
                         while let Some(i) =
                             instrs.next().expect("Can parse next CFI instruction OK")
                         {
@@ -561,9 +594,9 @@ mod cfi {
                     }
                     CieOrFde::Fde(partial) => {
                         let fde = partial
-                            .parse(|offset| eh_frame.cie_from_offset(&bases, offset))
+                            .parse(EhFrame::cie_from_offset)
                             .expect("Should be able to get CIE for FED");
-                        let mut instrs = fde.instructions();
+                        let mut instrs = fde.instructions(&eh_frame, &bases);
                         while let Some(i) =
                             instrs.next().expect("Can parse next CFI instruction OK")
                         {
@@ -580,9 +613,12 @@ mod cfi {
         let eh_frame = read_section("eh_frame");
         let eh_frame = EhFrame::new(&eh_frame, LittleEndian);
 
-        let bases = BaseAddresses::default().set_cfi(0).set_data(0).set_text(0);
+        let bases = BaseAddresses::default()
+            .set_eh_frame(0)
+            .set_got(0)
+            .set_text(0);
 
-        let mut ctx = Some(UninitializedUnwindContext::new());
+        let mut ctx = UninitializedUnwindContext::new();
 
         b.iter(|| {
             let mut entries = eh_frame.entries(&bases);
@@ -591,53 +627,48 @@ mod cfi {
                     CieOrFde::Cie(_) => {}
                     CieOrFde::Fde(partial) => {
                         let fde = partial
-                            .parse(|offset| eh_frame.cie_from_offset(&bases, offset))
+                            .parse(EhFrame::cie_from_offset)
                             .expect("Should be able to get CIE for FED");
-
-                        let mut context = ctx.take()
-                            .unwrap()
-                            .initialize(fde.cie())
+                        let mut table = fde
+                            .rows(&eh_frame, &bases, &mut ctx)
                             .expect("Should be able to initialize ctx");
-
+                        while let Some(row) =
+                            table.next_row().expect("Should get next unwind table row")
                         {
-                            let mut table = UnwindTable::new(&mut context, &fde);
-                            while let Some(row) =
-                                table.next_row().expect("Should get next unwind table row")
-                            {
-                                test::black_box(row);
-                            }
+                            test::black_box(row);
                         }
-
-                        ctx = Some(context.reset());
                     }
                 };
             }
         });
     }
 
-    fn instrs_len<R: Reader>(fde: &FrameDescriptionEntry<EhFrame<R>, R, R::Offset>) -> usize {
-        fde.instructions()
-            .fold(0, |count, _| count + 1)
+    fn instrs_len<R: Reader>(
+        eh_frame: &EhFrame<R>,
+        bases: &BaseAddresses,
+        fde: &FrameDescriptionEntry<R>,
+    ) -> usize {
+        fde.instructions(eh_frame, bases)
+            .fold(0, |count, _| Ok(count + 1))
             .expect("fold over instructions OK")
     }
 
     fn get_fde_with_longest_cfi_instructions<R: Reader>(
         eh_frame: &EhFrame<R>,
-    ) -> FrameDescriptionEntry<EhFrame<R>, R, R::Offset> {
-        let bases = BaseAddresses::default().set_cfi(0).set_data(0).set_text(0);
+        bases: &BaseAddresses,
+    ) -> FrameDescriptionEntry<R> {
+        let mut longest: Option<(usize, FrameDescriptionEntry<_>)> = None;
 
-        let mut longest: Option<(usize, FrameDescriptionEntry<_, _, _>)> = None;
-
-        let mut entries = eh_frame.entries(&bases);
+        let mut entries = eh_frame.entries(bases);
         while let Some(entry) = entries.next().expect("Should parse CFI entry OK") {
             match entry {
                 CieOrFde::Cie(_) => {}
                 CieOrFde::Fde(partial) => {
                     let fde = partial
-                        .parse(|offset| eh_frame.cie_from_offset(&bases, offset))
+                        .parse(EhFrame::cie_from_offset)
                         .expect("Should be able to get CIE for FED");
 
-                    let this_len = instrs_len(&fde);
+                    let this_len = instrs_len(eh_frame, bases, &fde);
 
                     let found_new_longest = match longest {
                         None => true,
@@ -658,10 +689,14 @@ mod cfi {
     fn parse_longest_fde_instructions(b: &mut test::Bencher) {
         let eh_frame = read_section("eh_frame");
         let eh_frame = EhFrame::new(&eh_frame, LittleEndian);
-        let fde = get_fde_with_longest_cfi_instructions(&eh_frame);
+        let bases = BaseAddresses::default()
+            .set_eh_frame(0)
+            .set_got(0)
+            .set_text(0);
+        let fde = get_fde_with_longest_cfi_instructions(&eh_frame, &bases);
 
         b.iter(|| {
-            let mut instrs = fde.instructions();
+            let mut instrs = fde.instructions(&eh_frame, &bases);
             while let Some(i) = instrs.next().expect("Should parse instruction OK") {
                 test::black_box(i);
             }
@@ -672,14 +707,17 @@ mod cfi {
     fn eval_longest_fde_instructions_new_ctx_everytime(b: &mut test::Bencher) {
         let eh_frame = read_section("eh_frame");
         let eh_frame = EhFrame::new(&eh_frame, LittleEndian);
-        let fde = get_fde_with_longest_cfi_instructions(&eh_frame);
+        let bases = BaseAddresses::default()
+            .set_eh_frame(0)
+            .set_got(0)
+            .set_text(0);
+        let fde = get_fde_with_longest_cfi_instructions(&eh_frame, &bases);
 
         b.iter(|| {
-            let mut ctx = UninitializedUnwindContext::new()
-                .initialize(fde.cie())
+            let mut ctx = UninitializedUnwindContext::new();
+            let mut table = fde
+                .rows(&eh_frame, &bases, &mut ctx)
                 .expect("Should initialize the ctx OK");
-
-            let mut table = UnwindTable::new(&mut ctx, &fde);
             while let Some(row) = table.next_row().expect("Should get next unwind table row") {
                 test::black_box(row);
             }
@@ -690,24 +728,21 @@ mod cfi {
     fn eval_longest_fde_instructions_same_ctx(b: &mut test::Bencher) {
         let eh_frame = read_section("eh_frame");
         let eh_frame = EhFrame::new(&eh_frame, LittleEndian);
-        let fde = get_fde_with_longest_cfi_instructions(&eh_frame);
+        let bases = BaseAddresses::default()
+            .set_eh_frame(0)
+            .set_got(0)
+            .set_text(0);
+        let fde = get_fde_with_longest_cfi_instructions(&eh_frame, &bases);
 
-        let mut ctx = Some(UninitializedUnwindContext::new());
+        let mut ctx = UninitializedUnwindContext::new();
 
         b.iter(|| {
-            let mut context = ctx.take()
-                .unwrap()
-                .initialize(fde.cie())
-                .expect("Should be able to initialize ctx");
-
-            {
-                let mut table = UnwindTable::new(&mut context, &fde);
-                while let Some(row) = table.next_row().expect("Should get next unwind table row") {
-                    test::black_box(row);
-                }
+            let mut table = fde
+                .rows(&eh_frame, &bases, &mut ctx)
+                .expect("Should initialize the ctx OK");
+            while let Some(row) = table.next_row().expect("Should get next unwind table row") {
+                test::black_box(row);
             }
-
-            ctx = Some(context.reset());
         });
     }
 }
